from .base_provider import BaseProvider
from unionllm.utils import ModelResponse, Message, Choices, Usage, Delta, StreamingChoices
import requests
import json
import logging, os

class MinimaxOpenAIError(Exception):
    def __init__(
        self,
        status_code,
        message,
    ):
        self.status_code = status_code
        self.message = message
        super().__init__(self.message)

class MinimaxAIProvider(BaseProvider):
    def __init__(self, **model_kwargs):
        # Get MINIMAX_API_KEY from environment variables
        _env_api_key = os.environ.get("MINIMAX_API_KEY")
        self.api_key = model_kwargs.get("api_key") if model_kwargs.get("api_key") else _env_api_key
        if not self.api_key:
            raise MinimaxOpenAIError(
                status_code=422, message=f"Missing API key"
            )
        self.endpoint_url = "https://api.minimax.chat/v1/text/chatcompletion_v2"

    def pre_processing(self, **kwargs):
        supported_params = [
            "model", "messages", "max_tokens", "temperature", "top_p", "n",
            "logprobs", "stream", "stop", "presence_penalty", "frequency_penalty",
            "best_of", "logit_bias"
        ]
        for key in list(kwargs.keys()):
            if key not in supported_params:
                kwargs.pop(key)
        return kwargs

    def post_stream_processing_wrapper(self, model, messages, **new_kwargs):
        payload = json.dumps({"model": model, "messages": messages, **new_kwargs})
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }
        response = requests.post(self.endpoint_url, headers=headers, data=payload)
        for line in response.iter_lines():
            if line:
                new_line = line.decode("utf-8").replace("data: ", "")
                data = json.loads(new_line)
                choices = data.get("choices", [])
                chunk_choices = []
                if choices:
                    for choice in choices:
                        chunk_delta = Delta()
                        delta = choice.get("delta")
                        if delta:
                            if "role" in choice['delta']:
                                chunk_delta.role = choice['delta']["role"]
                            if "content" in choice['delta']:
                                chunk_delta.content = choice['delta']["content"]
                            chunk_choices.append(StreamingChoices(index=choice['index'], delta=chunk_delta))
                    if "usage" in data:
                        chunk_usage = Usage()
                        if "total_tokens" in data["usage"]:
                            chunk_usage.total_tokens = data["usage"]["total_tokens"]

                chunk_response = ModelResponse(
                    id=data["id"],
                    choices=chunk_choices,
                    created=data["created"],
                    model=model,
                    usage=chunk_usage if "usage" in data else None,
                    stream=True
                )
                yield chunk_response

    def create_model_response_wrapper(self, result, model):
        response_dict = result.json()
        choices = []
        for choice in response_dict["choices"]:
            message = Message(
                content=choice["message"]["content"],
                role=choice["message"]["role"]
            )
            choices.append(
                Choices(
                    message=message,
                    index=choice["index"],
                    finish_reason=choice["finish_reason"],
                )
            )
        usage = Usage(
            total_tokens=response_dict["usage"]["total_tokens"],
        )
        response = ModelResponse(
            id=response_dict["id"],
            choices=choices,
            created=response_dict["created"],
            model=model,
            usage=usage,
        )
        return response

    def completion(self, model: str, messages: list, **kwargs):
        try:
            if model is None or messages is None:
                raise MinimaxOpenAIError(
                    status_code=422, message=f"Missing model or messages"
                )
                
            message_check_result = self.check_prompt("minimax", model, messages)            
            if message_check_result['pass_check']:
                messages = message_check_result['messages']
            else:
                raise MinimaxOpenAIError(
                    status_code=422, message=message_check_result['reason']
                )
                
            new_kwargs = self.pre_processing(**kwargs)
            stream = kwargs.get("stream", False)

            if stream:
                return self.post_stream_processing_wrapper(model=model, messages=messages, **new_kwargs)
            
            else:
                payload = json.dumps({"model": model, "messages": messages, **new_kwargs})
                headers = {
                    "Authorization": f"Bearer {self.api_key}",
                    "Content-Type": "application/json",
                }
                result = requests.post(self.endpoint_url, headers=headers, data=payload)
                return self.create_model_response_wrapper(result, model=model)
        except Exception as e:
            if hasattr(e, "status_code"):
                raise MinimaxOpenAIError(status_code=e.status_code, message=str(e))
            else:
                raise MinimaxOpenAIError(status_code=500, message=str(e))


from abc import ABC, abstractmethod
from ..models import ResponseModel
from typing import TYPE_CHECKING, Any, Dict, Optional, Sequence, List
from unionllm.utils import ModelResponse, Message, Choices, Usage, Context, StreamingChoices, Delta, check_object_input_support, check_video_input_support, check_vision_input_support, reformat_object_content, check_file_input_support

import openai
import json
from openai._models import BaseModel as OpenAIObject

# if TYPE_CHECKING:
from dataclasses import dataclass

@dataclass
class BaseProvider(ABC):
    args: Optional[Dict[str, Any]] = None
    key: Optional[str] = None
    group: Optional[str] = None

    def __init__(
        self,
        status_code,
        message,
    ):
        self.status_code = status_code
        self.message = message

    @abstractmethod
    def completion(self, model: str, messages: list) -> ResponseModel:
        pass

    def check_prompt(self, provider, model, messages):
        # 遍历messages列表，判断消息中间是否存在system消息，是否所有消息content都是string类型, 是否消息中包含图片和文件类型
        is_invalid_format = False
        has_middle_system = False
        has_object_content = False
        has_vision_input = False
        has_video_input = False
        has_file_input = False
        for i, message in enumerate(messages):
            if message.get("role") == "system":
                if i != 0:
                    has_middle_system = True
            if not isinstance(message.get("content"), str):
                if isinstance(message.get("content"), list):
                    message_contents = message.get("content")
                    for content in message_contents:
                        if not isinstance(content, dict):
                            is_invalid_format = True
                        else:
                            has_object_content = True
                            if content.get("type") == "image_url":
                                if content.get("image_url").get("url"):
                                    has_vision_input = True
                                else:
                                    is_invalid_format = True
                            elif content.get("type") == "video_url":
                                if content.get("video_url").get("url"):
                                    has_video_input = True
                                else:
                                    is_invalid_format = True
                            elif content.get("type") == "file_url":
                                if content.get("file_url").get("url"):
                                    has_file_input = True
                                else:
                                    is_invalid_format = True
                                                                      
        if is_invalid_format:
            return {"pass_check": False, "reformatted": False, "reason": "Invalid message format"}

        reformated = 0
        if has_object_content:
            # 判断object content是否支持
            object_support = check_object_input_support(provider)
            if object_support == "NONE":
                # 如果不支持
                if has_vision_input or has_file_input:
                    # 如果包含图片或文件，则返回错误信息
                    return {"pass_check": False, "reformatted": False, "reason": "Object content is not supported"}
                else:
                    # 如果不包含，则将object content转为文本
                    messages = reformat_object_content(messages, False, False, False)
                    reformated = 1
            else:
                # 如果支持
                reformat_image = 0
                reformat_file = 0
                reformat_video = 0
                if object_support == "PARTIAL":
                    # 如果部分支持，则需要将object content转为文本
                    reformated = 1
                if has_vision_input:
                    # 如果包含图片
                    vision_input_support = check_vision_input_support(provider, model)
                    if vision_input_support == "PARTIAL":
                        # 如果图片是部分支持，则需要将图片转为文本
                        reformat_image = 1
                        reformated = 1
                    elif vision_input_support == "NONE":
                        # 如果图片不支持，则返回错误信息
                        return {"pass_check": False, "reformatted": False, "reason": "Vision input is not supported"}
                
                if has_video_input:
                    video_input_support = check_video_input_support(provider, model)
                    if video_input_support == "PARTIAL":
                        # 如果视频是部分支持，则需要将视频转为文本
                        reformat_video = 1
                        reformated = 1
                    elif video_input_support == "NONE":
                        # 如果视频不支持，则返回错误信息
                        return {"pass_check": False, "reformatted": False, "reason": "Video input is not supported"}
                    
                if has_file_input:
                    # 如果包含文件
                    file_input_support = check_file_input_support(provider, model)
                    if file_input_support == "PARTIAL":
                        # 如果文件是部分支持，则需要将文件转为文本
                        reformat_file= 1
                        reformated = 1
                    elif file_input_support == "NONE":
                        # 如果文件不支持，则返回错误信息
                        return {"pass_check": False, "reformatted": False, "messages": messages, "reason": "File input is not supported"}
                if reformated or reformat_image or reformat_file or reformat_video:
                    messages = reformat_object_content(messages, 1, reformat_image, reformat_file, reformat_video)
                
        return {"pass_check": True, "reformatted": reformated, "messages": messages}         

    def create_model_response(
        self, openai_response: openai.ChatCompletion, model: str
    ) -> ModelResponse:
        choices = []
        
        for choice in openai_response.choices:
            # 假设choice.message.tool_calls存在
            tool_calls = getattr(choice.message, 'tool_calls', None)

            # 创建Message对象时，仅当tool_calls存在时才添加tool_calls属性
            if tool_calls:
                message = Message(content=choice.message.content, role=choice.message.role, tool_calls=tool_calls)
            else:
                message = Message(content=choice.message.content, role=choice.message.role)
            choices.append(
                Choices(
                    message=message, index=choice.index, finish_reason=choice.finish_reason
                )
            )

        # 把Usage对象里面的其他属性也添加到Usage对象中
        usage_dict = openai_response.usage.model_dump()
        usage = Usage(**usage_dict)

        response = ModelResponse(
            id=openai_response.id,
            choices=choices,
            created=openai_response.created,
            model=model,
            usage=usage,
        )
        return response
    
    def post_stream_processing(self, response, model=None):
        for chunk in response:
            data = chunk.json()
            # print(data)
            if isinstance(data, str):
                data = json.loads(data)
            if 'choices' in data:
                choices = data['choices']
                chunk_choices = []
                for choice in choices:
                    # 判断如果choice是StreamingChoices类型的对象，则直接添加到chunk_choices中
                    if isinstance(choice, StreamingChoices):
                        chunk_choices.append(choice)
                    else:
                        delta = choice.get("delta")
                        if choice.get("finish_reason") == "stop":
                            stream_choices = StreamingChoices(index=choice['index'], finish_reason="stop")
                            chunk_choices.append(stream_choices)
                        elif delta:
                            delta = choice.get("delta")
                            chunk_delta = Delta()
                            if "role" in choice['delta']:
                                chunk_delta.role = choice['delta']["role"]
                            if "content" in choice['delta']:
                                chunk_delta.content = choice['delta']["content"]
                            if "tool_calls" in choice['delta']:
                                chunk_delta.tool_calls = choice['delta']["tool_calls"]
                            
                            stream_choices = StreamingChoices(index=choice['index'], delta=chunk_delta, finish_reason=choice.get("finish_reason"))
                            # 遍历choices字典中的key, 如果在特定的列表中，则添加到stream_choices中
                            for key in choice.keys():
                                if key in ["content_filter_results", "content_filter_offsets", "logprobs"]:
                                    setattr(stream_choices, key, choice[key])
                            chunk_choices.append(stream_choices)
                    
            if "usage" in data:
                chunk_usage = Usage()
                if data["usage"]:
                    # 把usage字典中的数据添加到chunk_usage中
                    for key, value in data["usage"].items():
                        setattr(chunk_usage, key, value)
            
            chunk_response = ModelResponse(
                id=data["id"],
                choices=chunk_choices,
                created=data["created"],
                model=model,
                usage=chunk_usage if "usage" in data else None,
                stream=True,
                system_fingerprint=data.get("system_fingerprint") if "system_fingerprint" in data else None
            )
            yield chunk_response


import logging
import asyncio
from functools import partial

from typing import Any, List
from .providers import zhipu, moonshot, xai, minimax, qwen, tiangong, baichuan, wenxin, xunfei, xunfei_http, dify, fastgpt, coze, litellm, lingyi, stepfun, doubao, deepseek
from .exceptions import ProviderError
# from litellm import completion as litellm_completion
from typing import Optional

logger = logging.getLogger(__name__)

class UnionLLM:
    def __init__(self, provider: Optional[str] = None, **kwargs):
        self.provider = provider.lower() if provider else None
        self.litellm_call_type = None
        if self.provider == "zhipuai":
            self.provider_instance = zhipu.ZhipuAIProvider(**kwargs)
        elif self.provider == "moonshot":
            self.provider_instance = moonshot.MoonshotAIProvider(**kwargs)
        elif self.provider == "minimax":
            self.provider_instance = minimax.MinimaxAIProvider(**kwargs)
        elif self.provider == "qwen":
            self.provider_instance = qwen.QwenAIProvider(**kwargs)
        elif self.provider == "tiangong":
            self.provider_instance = tiangong.TianGongAIProvider(**kwargs)
        elif self.provider == "baichuan":
            self.provider_instance = baichuan.BaiChuanAIProvider(**kwargs)
        elif self.provider == "wenxin":
            self.provider_instance = wenxin.WenXinAIProvider(**kwargs)
        elif self.provider == "xunfei":
            self.provider_instance = xunfei.XunfeiAIProvider(**kwargs)
        elif self.provider == "xunfei_http":
            self.provider_instance = xunfei_http.XunfeiHTTPProvider(**kwargs)
        # elif self.provider == "xai":
        #     self.provider_instance = xai.XAIHTTPProvider(**kwargs)
        elif self.provider == "dify":
            self.provider_instance = dify.DifyAIProvider(**kwargs)
        elif self.provider == "fastgpt":
            self.provider_instance = fastgpt.FastGPTProvider(**kwargs)
        elif self.provider == "coze":
            self.provider_instance = coze.CozeAIProvider(**kwargs)
        elif self.provider == "lingyi":
            self.provider_instance = lingyi.LingyiAIProvider(**kwargs)
        elif self.provider == "stepfun":
            self.provider_instance = stepfun.StepfunAIProvider(**kwargs)
        elif self.provider == "doubao":
            self.provider_instance = doubao.DouBaoAIProvider(**kwargs)
        # elif self.provider == "deepseek":
        #     self.provider_instance = deepseek.DeepSeekAIProvider(**kwargs)
        elif self.provider:
            if_litellm_support, support_type = self.check_litellm_providers(provider=self.provider)
            if if_litellm_support:
                self.provider_instance = litellm.LiteLLMProvider(**kwargs)
                if support_type == 1:
                    self.litellm_call_type = 1
                elif support_type == 2:
                    self.litellm_call_type = 2
                elif support_type == 3:
                    self.litellm_call_type = 3
            else:
                raise ProviderError(f"Provider '{self.provider}' is not supported.")
        else:
            self.provider_instance = litellm.LiteLLMProvider(**kwargs)

    def completion(self, model: str, messages: List[str], **kwargs) -> Any:
        if not self.provider_instance:
            raise ProviderError(f"Provider '{self.provider}' is not initialized.")
        if self.litellm_call_type:
            if self.litellm_call_type == 1:
                # Jugde whether the model starts with self.provider, if not, add it
                if not model.startswith(self.provider+"/"):
                    model = f"{self.provider}/{model}"     
                return self.provider_instance.completion(model, messages, **kwargs)
            elif self.litellm_call_type == 2:
                return self.provider_instance.completion(model, messages, **kwargs)
            elif self.litellm_call_type == 3:
                # append OpenAI as provider name
                if self.provider == 'xai':
                    kwargs['api_base'] = "https://api.x.ai/v1"
                model = f"openai/{model}"
                return self.provider_instance.completion(model, messages, **kwargs)
        else:
            return self.provider_instance.completion(model, messages, **kwargs)
        
    async def acompletion(self, model: str, messages: List[str], **kwargs) -> Any:
        loop = asyncio.get_event_loop()
        if not self.provider_instance:
            raise ProviderError(f"Provider '{self.provider}' is not initialized.")
        
        if self.litellm_call_type:
            if self.litellm_call_type == 1:
                if not model.startswith(self.provider + "/"):
                    model = f"{self.provider}/{model}"
                func = partial(self.provider_instance.completion, model, messages, **kwargs)
                return await loop.run_in_executor(None, func)
            elif self.litellm_call_type == 2:
                func = partial(self.provider_instance.completion, model, messages, **kwargs)
                return await loop.run_in_executor(None, func)
        else:
            func = partial(self.provider_instance.completion, model, messages, **kwargs)
            return await loop.run_in_executor(None, func)

    def check_litellm_providers(self, provider: str) -> bool:
        # Judge whether the provider is supported by LiteLLM, and if provider name should be added to the model name
        if provider in ['azure', 'anthropic', 'deepseek', 'sagemaker', 'bedrock', 'vertex_ai', 'vertex_ai_beta', 'palm', 'gemini', 'mistral', 'cloudflare', 'huggingface', 'replicate', 'together_ai', 'openrouter', 'baseten', 'nlp_cloud', 'petals', 'ollama', 'perplexity', 'groq', 'anyscale', 'watsonx', 'voyage', 'xinference']:
            # provider name should be added to the model name
            return True, 1
        elif provider in ['openai', 'cohere', 'ai21', 'deepinfra', 'ai21', 'alpha_alpha']:
            # provider name should not be added to the model name
            return True, 2
        elif provider in ['xai']:
            # append OpenAI as procall as OpenAI campatible API
            return True, 3
        else:
            return False, 0
        
# This is the new function to simplify calling
def unionchat(model: str, messages: List[dict], **kwargs) -> Any:
    client = UnionLLM(**kwargs)
    return client.completion(model=model, messages=messages, **kwargs)


from .base_provider import BaseProvider
from unionllm.utils import ModelResponse, Message, Choices, Usage, Context, Delta, StreamingChoices
from openai import OpenAI
import logging, json, time, requests, os


class FastGPTError(Exception):
    def __init__(
        self,
        status_code,
        message,
    ):
        self.status_code = status_code
        self.message = message
        super().__init__(self.message)


class FastGPTProvider(BaseProvider):
    def __init__(self, **model_kwargs):
        # Get FASTGPT_API_KEY from environment variables
        _env_api_key = os.environ.get("FASTGPT_API_KEY")
        self.api_key = model_kwargs.get("api_key") if model_kwargs.get("api_key") else _env_api_key
        if not self.api_key:
            raise FastGPTError(
                status_code=422, message=f"Missing API key"
            )
        self.base_url = model_kwargs.get("api_base") if model_kwargs.get("api_base") else "https://api.fastgpt.in/api/v1"
        self.endpoint_url = self.base_url + "/chat/completions"

    def pre_processing(self, **kwargs):
        # process the compatibility issue of parameters, all unsupported parameters are discarded
        supported_params = [
            "model", "messages", "max_tokens", "temperature", "top_p", "n",
            "logprobs", "stream", "stop", "presence_penalty", "frequency_penalty",
            "best_of", "logit_bias"
        ]
        for key in list(kwargs.keys()):
            if key not in supported_params:
                kwargs.pop(key)
        # 获取详细信息，如果希望得到RAG的Context则必须开启
        kwargs.update({"detail": True})
        return kwargs

    def post_stream_processing_wrapper(self, model, messages, **new_kwargs):
        payload = json.dumps({"model": model, "messages": messages, **new_kwargs})
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }
        response = requests.post(self.endpoint_url, headers=headers, data=payload)
        index = 0
        # 解析stream返回信息并生成OpenAI兼容格式
        for line in response.iter_lines():
            if line:
                chunk_choices = []
                chunk_context = []
                if line.startswith(b"data:"):
                    new_line = line.decode("utf-8").replace("data: ", "")
                    if new_line == "[DONE]":
                        continue
                    try:
                        data = json.loads(new_line)
                    except Exception as e:
                        continue

                    if "choices" in data:
                        chunk_message = data["choices"][0]["delta"]
                        chunk_delta = Delta()
                        if chunk_message:
                            if "role" in chunk_message:
                                chunk_delta.role = chunk_message['role']
                            if "content" in chunk_message:
                                chunk_delta.content = chunk_message['content']
                            chunk_choices.append(StreamingChoices(index=str(index), delta=chunk_delta))

                    if isinstance(data, list):
                        summary = data[0]
                        if "tokens" in summary:
                            chunk_usage = Usage()
                            chunk_usage.total_tokens = summary['tokens']
                        else:
                            chunk_usage = None
                    else:
                        chunk_usage = None

                    if isinstance(data, list):
                        for module in data:
                            if "quoteList" in module:
                                for quote in module["quoteList"]:
                                    content = f'question:[{quote["q"]}], answer:[{quote["a"]}]'
                                    chunk_context.append(
                                        {
                                            "id": quote["id"],
                                            "content": content,
                                        }    
                                    )
                
                    chunk_response = ModelResponse(
                        id=data['id'] if 'id' in data else None,
                        choices=chunk_choices,
                        context=chunk_context,
                        created=int(time.time()),
                        model=model,
                        usage=chunk_usage if chunk_usage else None,
                        stream=True
                    )
                    index += 1
                    yield chunk_response

    def create_model_response_wrapper(self, result):
        response_dict = result.json()
        choices = []
        context = []
        message = Message(
            content=response_dict['choices'][0]['message']['content'],
            role="assistant"
        )
        choices.append(
            Choices(
                message=message,
                index=0,
                finish_reason="stop",
            )
        )

        usage = Usage(
            prompt_tokens=response_dict['usage']['prompt_tokens'],
            completion_tokens=response_dict['usage']['completion_tokens'],
            total_tokens=response_dict['usage']['total_tokens'],
        )

        # 解析quoteList中包含的q&a背景信息并合成为字符串
        if "responseData" in response_dict:
            for module in response_dict["responseData"]:
                if "quoteList" in module:
                    for quote in module["quoteList"]:
                        content = f'question:[{quote["q"]}], answer:[{quote["a"]}]'
                        context.append(Context(id=quote["id"], content=content))

        response = ModelResponse(
            id=response_dict.get("id", ""),
            choices=choices,
            context=context,
            created=int(time.time()),
            model='',
            usage=usage,
        )
        return response

    def completion(self, model: str, messages: list, **kwargs):
        try:
            if model is None or messages is None:
                raise FastGPTError(
                    status_code=422, message="Missing model or messages"
                )
                
            message_check_result = self.check_prompt("fastgpt", model, messages)    
            
            if message_check_result['pass_check']:
                messages = message_check_result['messages']
            else:
                raise FastGPTError(
                    status_code=422, message=message_check_result['reason']
                )
                
            new_kwargs = self.pre_processing(**kwargs)
            stream = kwargs.get("stream", False)

            if stream:
                return self.post_stream_processing_wrapper(model, messages, **new_kwargs)
            else:
                payload = json.dumps({"model": model, "messages": messages, **new_kwargs})
                headers = {
                    "Authorization": f"Bearer {self.api_key}",
                    "Content-Type": "application/json",
                }
                result = requests.post(self.endpoint_url, headers=headers, data=payload)
                return self.create_model_response_wrapper(result)
        except Exception as e:
            if hasattr(e, "status_code"):
                raise FastGPTError(status_code=e.status_code, message=str(e))
            else:
                raise FastGPTError(status_code=500, message=str(e))


import json
from abc import ABC, abstractmethod
from openai import OpenAIError as OriginalError
from typing import List, Union, Optional

import uuid, time, openai, random, requests
from openai._models import BaseModel as OpenAIObject

class Message(OpenAIObject):
    def __init__(
        self,
        content="default",
        role="assistant",
        logprobs=None,
        function_call=None,
        **params
    ):
        super(Message, self).__init__(**params)
        self.content = content
        self.role = role
        self._logprobs = logprobs
        if function_call:
            self.function_call = function_call

    def get(self, key, default=None):
        # Custom .get() method to access attributes with a default value if the attribute doesn't exist
        return getattr(self, key, default)

    def __getitem__(self, key):
        # Allow dictionary-style access to attributes
        return getattr(self, key)

    def __setitem__(self, key, value):
        # Allow dictionary-style assignment of attributes
        setattr(self, key, value)


class Delta(OpenAIObject):
    def __init__(self, content=None, role=None, **params):
        super(Delta, self).__init__(**params)
        if content is not None:
            self.content = content
        if role:
            self.role = role

    def __contains__(self, key):
        # Define custom behavior for the 'in' operator
        return hasattr(self, key)

    def get(self, key, default=None):
        # Custom .get() method to access attributes with a default value if the attribute doesn't exist
        return getattr(self, key, default)

    def __getitem__(self, key):
        # Allow dictionary-style access to attributes
        return getattr(self, key)

    def __setitem__(self, key, value):
        # Allow dictionary-style assignment of attributes
        setattr(self, key, value)


class Choices(OpenAIObject):
    def __init__(self, finish_reason=None, index=0, message=None, **params):
        super(Choices, self).__init__(**params)
        if finish_reason:
            self.finish_reason = map_finish_reason(finish_reason)
        else:
            self.finish_reason = "stop"
        self.index = index
        if message is None:
            self.message = Message(content=None)
        else:
            self.message = message

    def __contains__(self, key):
        # Define custom behavior for the 'in' operator
        return hasattr(self, key)

    def get(self, key, default=None):
        # Custom .get() method to access attributes with a default value if the attribute doesn't exist
        return getattr(self, key, default)

    def __getitem__(self, key):
        # Allow dictionary-style access to attributes
        return getattr(self, key)

    def __setitem__(self, key, value):
        # Allow dictionary-style assignment of attributes
        setattr(self, key, value)

class Usage(OpenAIObject):
    def __init__(
        self, prompt_tokens=None, completion_tokens=None, total_tokens=None, **params
    ):
        super(Usage, self).__init__(**params)
        if prompt_tokens:
            self.prompt_tokens = prompt_tokens
        if completion_tokens:
            self.completion_tokens = completion_tokens
        if total_tokens:
            self.total_tokens = total_tokens
        
    def __contains__(self, key):
        # Define custom behavior for the 'in' operator
        return hasattr(self, key)

    def get(self, key, default=None):
        # Custom .get() method to access attributes with a default value if the attribute doesn't exist
        return getattr(self, key, default)

    def __getitem__(self, key):
        # Allow dictionary-style access to attributes
        return getattr(self, key)

    def __setitem__(self, key, value):
        # Allow dictionary-style assignment of attributes
        setattr(self, key, value)

class Context(OpenAIObject):
    def __init__(self, id=None, content=None, score=None, **params):
        super(Context, self).__init__(**params)
        if id:
            self.id = id
        if content:
            self.content = content
        if score:
            self.score = score

    def __contains__(self, key):
        # Define custom behavior for the 'in' operator
        return hasattr(self, key)

    def get(self, key, default=None):
        # Custom .get() method to access attributes with a default value if the attribute doesn't exist
        return getattr(self, key, default)

    def __getitem__(self, key):
        # Allow dictionary-style access to attributes
        return getattr(self, key)

    def __setitem__(self, key, value):
        # Allow dictionary-style assignment of attributes
        setattr(self, key, value)

class StreamingChoices(OpenAIObject):
    def __init__(
        self, finish_reason=None, index=0, delta: Optional[Delta] = None, **params
    ):
        super(StreamingChoices, self).__init__(**params)

        self.finish_reason = finish_reason
        self.index = index
        if delta:
            self.delta = delta
        else:
            self.delta = Delta()

    def __contains__(self, key):
        # Define custom behavior for the 'in' operator
        return hasattr(self, key)

    def get(self, key, default=None):
        # Custom .get() method to access attributes with a default value if the attribute doesn't exist
        return getattr(self, key, default)

    def __getitem__(self, key):
        # Allow dictionary-style access to attributes
        return getattr(self, key)

    def __setitem__(self, key, value):
        # Allow dictionary-style assignment of attributes
        setattr(self, key, value)


def _generate_id():  # private helper function
    return "chatcmpl-" + str(uuid.uuid4())


def map_finish_reason(
    finish_reason: str,
):  # openai supports 5 stop sequences - 'stop', 'length', 'function_call', 'content_filter', 'null'
    # anthropic mapping
    if finish_reason == "stop_sequence":
        return "stop"
    return finish_reason


class ModelResponse(OpenAIObject):
    id: str
    """A unique identifier for the completion."""

    choices: List[Union[Choices, StreamingChoices]]
    """The list of completion choices the model generated for the input prompt."""

    created: int
    """The Unix timestamp (in seconds) of when the completion was created."""

    model: Optional[str] = None
    """The model used for completion."""

    object: str
    """The object type, which is always "text_completion" """

    system_fingerprint: Optional[str] = None
    """This fingerprint represents the backend configuration that the model runs with.

    Can be used in conjunction with the `seed` request parameter to understand when
    backend changes have been made that might impact determinism.
    """

    usage: Optional[Usage] = None
    """Usage statistics for the completion request."""  

    _hidden_params: dict = {}

    def __init__(
        self,
        id=None,
        choices=None,
        created=None,
        model=None,
        object=None,
        system_fingerprint=None,
        usage=None,
        conversation_id=None,
        stream=False,
        response_ms=None,
        hidden_params=None,
        **params
    ):
        if stream:
            object = "chat.completion.chunk"
        else:
            object = "chat.completion"
        if id is None:
            id = _generate_id()
        if created is None:
            created = int(time.time())
        if response_ms:
            _response_ms = response_ms
        if not usage:
            usage = Usage()
        
        if hidden_params:
            self._hidden_params = hidden_params

        filtered_params = {}      
        if conversation_id:
            filtered_params['conversation_id'] = conversation_id

        # 只有当 conversation_id 不为 None 时才添加到 filtered_params
        if 'conversation_id'in params and conversation_id is not None:
            filtered_params['conversation_id'] = conversation_id
        
        super().__init__(
            id=id,
            choices=choices,
            created=created,
            model=model,
            object=object,
            system_fingerprint=system_fingerprint,
            usage=usage,
            **filtered_params
        )

    def __contains__(self, key):
        # Define custom behavior for the 'in' operator
        return hasattr(self, key)

    def get(self, key, default=None):
        # Custom .get() method to access attributes with a default value if the attribute doesn't exist
        return getattr(self, key, default)

    def __getitem__(self, key):
        # Allow dictionary-style access to attributes
        return getattr(self, key)

    def __setitem__(self, key, value):
        # Allow dictionary-style assignment of attributes
        setattr(self, key, value)

class EmbeddingResponse(OpenAIObject):
    def __init__(
        self,
        id=None,
        choices=None,
        created=None,
        model=None,
        usage=None,
        stream=False,
        response_ms=None,
        **params
    ):
        self.object = "list"
        if response_ms:
            self._response_ms = response_ms
        else:
            self._response_ms = None
        self.data = []
        self.model = model

    def to_dict_recursive(self):
        d = super().to_dict_recursive()
        return d

def generate_unique_uid():
    # Get the current time in microseconds
    microseconds = int(time.time() * 1000000)
    
    # Get a random number between 0 and 0xFFFF
    rand_num = random.randint(0, 0xFFFF)
    
    # Combine them to get a unique ID
    unique_uid = f"{microseconds:x}{rand_num:04x}"
    
    return unique_uid

def check_object_input_support(provider):
    not_supported_providers = ["wenxin", "baichuan", "minimax", "xunfei", "tiangong", "lingyi", "fastgpt", "doubao", "moonshot"]
    if provider in not_supported_providers:
        return "NONE"
    elif provider == "coze":
        return "PARTIAL"
    else:
        return "FULL"

def check_file_input_support(provider, model):
    # supported_providers = ["openai", "anthropic", "claude"]
    if provider == "coze":
        return "PARTIAL"
    # elif provider in supported_providers:
    #     return "PARTIAL"
    else:
        return "PARTIAL"

def check_vision_input_support(provider, model):
    # supported_providers = ['zhipuai']
    if provider == "coze":
        return "PARTIAL"
    elif provider == "dify":
        return "FULL"
    else:
        # 默认支持，不做限制
        return "FULL"
    
    # supported_models = [
    #     {"zhipuai": ["glm-4v", "glm-4v-plus"]}
    # ]
    
    # for entry in supported_models:
    #     if provider in entry:
    #         if model in entry[provider]:
    #             return "FULL"    
    # return "NONE"

def check_video_input_support(provider, model):
    # supported_providers = ['zhipuai']
    if provider == "coze":
        return "PARTIAL"
    # elif provider not in supported_providers:
    #     return "NONE"
    else:
        # 默认支持，不做限制
        return "FULL"

    # supported_models = [
    #     {"zhipuai": ["glm-4v-plus"]}
    # ]
    
    # for entry in supported_models:
    #     if provider in entry:
    #         if model in entry[provider]:
    #             return "FULL"    
    # return "NONE"

def reformat_object_content(messages, reformat=False, reformat_image=False, reformat_file=False, reformat_video=False):
    formatted_messages = []
    for message in messages:
        new_formatted_message = {}
        if isinstance(message.get("content"), list):
            new_formatted_message["role"] = message.get("role")
            new_formatted_message["content"] = ""
            message_contents = message.get("content")            
            for content in message_contents:
                if not isinstance(content, dict):
                    return False
                content_type = content.get("type")
                if content_type == "text":
                    if isinstance(content.get("text"), str):
                        new_formatted_message["content"] += content.get("text")
                elif content_type in ["image_url","image"]:
                    if not reformat_image:
                        formatted_messages.append(message)
                        continue
                    elif content.get("image_url") and content.get("image_url").get("url"):
                        new_formatted_message["content"] += f"![image]({content.get('image_url').get('url')})"
                    else:
                        return False
                elif content_type == "video_url":
                    if not reformat_video:
                        formatted_messages.append(message)
                        continue
                    elif content.get("video_url") and content.get("video_url").get("url"):
                        new_formatted_message["content"] += f"![video]({content.get('video_url').get('url')})"
                    else:
                        return False
                elif content_type == "file_url":
                    if not reformat_file:
                        formatted_messages.append(message)
                        continue
                    elif content.get("file_url") and content.get("file_url").get("url"):
                        new_formatted_message["content"] += f"[file]({content.get('file_url').get('url')})"
                    else:
                        return False
            formatted_messages.append(new_formatted_message)   
        else:
            formatted_messages.append(message)
    return formatted_messages


from setuptools import setup, find_packages

setup(
    name='unionllm',
    version='0.1.23',
    license='MIT',
    description='A Python library for unified access to Chinese domestic large language models.',
    author='everfly',
    author_email='tagriver@gmail.com',
    url='https://github.com/EvalsOne/UnionLLM',
    packages=find_packages(exclude=['cookbook', 'tests', 'docs']),
    package_data={'': ['*.txt', '*.rst']},
    include_package_data=True,
    exclude_package_data={'': ['.DS_Store']},
    install_requires=[
        'openai',
        'pydantic',
        'zhipuai',
        'tenacity',
        'dashscope',
        'websocket_client',
        'requests',
        'litellm'
    ],
)